==> v0.2.0.tar.gz: shellfu/test/include/simple.sh <==
#!/bin/bash

. $TF_PROJDIR/test/include/common.sh

tf_enum_subtests() {
    #
    #
    #
    tf_warn "implement tf_enum_subtests()!"
}

tf_parse_meta() {
    #
    # Parse test meta-data and output a sh code
    #
    # Test header, i.e. all lines before first empty line
    # is searched for meta-data defined like
    #
    #     # key: value
    #
    # and values are converted to /bin/sh assignments that
    # can then be used for eval.
    #
    # Supported keys are `env`, `desc` and `cmd`.  One special
    # value is added called `start_at` that contains number of
    # forst line that is already a body.
    #
    perl -we '
        my @fnames = split ",", $ARGV[0];
        my $pfx = $ARGV[1];
        sub quote {
            my $s = $_[0];
            $s =~ s/\x27/\x27\\\x27\x27/g;
            return "\x27$s\x27";
        }
        sub parse_head {
            #
            # parse header and return meta-data
            #
            my $md = {};
            while (<STDIN>) {
                chomp;
                m/^$/ and last;
                my $line = $_;
                for (@fnames) {
                    $line =~ m/^# $_: (.*)/ and $$md{"$pfx$_"} = quote($1);
                }
            }
            $$md{"${pfx}start_at"} = $. + 1;
            return $md;
        }
        my $md = parse_head();
        print "$_=$$md{$_}\n" foreach keys %{$md};
    ' -- $1 $2
}

tf_do_subtest() {
    local name=$1
    if ! test -f "$name";
    then
        tf_warn "missing test data file: $name"
        return "$TF_ES_BAILOUT"
    fi
    touch $name.oracle
    touch $name.oracle.stderr
    touch $name.oracle.stdout
    if ! eval "$(tf_parse_meta desc,cmd,env t_ < $name)";
    then
        tf_warn "bad header format: $name";
        return $TF_ES_QUIT
    fi
    if ! eval "$(tf_parse_meta stdout,stderr,es o_ < $name.oracle)";
    then
        tf_warn "bad header format: $name";
        return $TF_ES_QUIT
    fi
    test -n "$o_stdout" && o_stdout=$name.oracle.stdout
    test -n "$o_stderr" && o_stderr=$name.oracle.stderr
    test -n "$o_es"     && o_es=0
    tf_debug "t_cmd='$t_cmd'"
    tf_debug "t_desc='$t_desc'"
    tf_debug "t_env='$t_env'"
    tf_debug "o_stdout='$o_stdout'"
    tf_debug "o_stderr='$o_stderr'"
    tf_debug "o_es='$o_es'"
    local tcmd="
        tail -n+$t_start_at $name \\
          | $t_env $t_cmd - > $name.result.stdout \\
          | 2> $name.result.stderr
    "
    eval "$tcmd"; es=$?
    diff -u $name.oracle.stderr $name.result.stderr || return $TF_ES_FAIL
    diff -u $name.oracle.stdout $name.result.stdout || return $TF_ES_FAIL
}

tf_do_subtests() {
    local subtname es tes
    es=0
    for subtname in $(tf_enum_subtests | grep -e "$TF_FILTER_SUBTEST");
    do
        tf_think "::: $TF_TNAME::$subtname"
        tf_do_subtest "$subtname";
        tes=$?
        test $tes -gt $es && es=$tes
        test $tes -gt 0 && tf_warn "!!! $TF_TNAME::$subtname ($tes)"
    done
    exit $es
}

==> v0.5.2.tar.gz: shellfu/test/include/simple.sh <==
#!/bin/bash

. $TF_PROJDIR/test/include/common.sh

tf_enum_subtests() {
    #
    #
    #
    tf_warn "implement tf_enum_subtests()!"
}

tf_parse_meta() {
    #
    # Parse test meta-data and output a sh code
    #
    # Test header, i.e. all lines before first empty line
    # is searched for meta-data defined like
    #
    #     # key: value
    #
    # and values are converted to /bin/sh assignments that
    # can then be used for eval.
    #
    # Supported keys are `env`, `desc` and `cmd`.  One special
    # value is added called `start_at` that contains number of
    # forst line that is already a body.
    #
    perl -we '
        my @fnames = split ",", $ARGV[0];
        my $pfx = $ARGV[1];
        sub quote {
            my $s = $_[0];
            $s =~ s/\x27/\x27\\\x27\x27/g;
            return "\x27$s\x27";
        }
        sub parse_head {
            #
            # parse header and return meta-data
            #
            my $md = {};
            while (<STDIN>) {
                chomp;
                m/^$/ and last;
                my $line = $_;
                for (@fnames) {
                    $line =~ m/^# $_: (.*)/ and $$md{"$pfx$_"} = quote($1);
                }
            }
            $$md{"${pfx}start_at"} = $. + 1;
            return $md;
        }
        my $md = parse_head();
        print "$_=$$md{$_}\n" foreach keys %{$md};
    ' -- $1 $2
}

tf_do_subtest() {
    local setup=true
    local cleanup=true
    test -f TF_SETUP   && setup="bash -e TF_SETUP"
    test -f TF_CLEANUP && cleanup="bash -e TF_CLEANUP"
    local name=$1
    if ! test -f "$name";
    then
        tf_warn "missing test data file: $name"
        return "$TF_ES_BAILOUT"
    fi
    touch $name.oracle
    touch $name.oracle.stderr
    touch $name.oracle.stdout
    if ! eval "$(tf_parse_meta desc,cmd,env t_ < $name)";
    then
        tf_warn "bad header format: $name";
        return $TF_ES_BAILOUT
    fi
    if ! eval "$(tf_parse_meta stdout,stderr,es o_ < $name.oracle)";
    then
        tf_warn "bad header format: $name";
        return $TF_ES_BAILOUT
    fi
    test -n "$o_stdout" && o_stdout=$name.oracle.stdout
    test -n "$o_stderr" && o_stderr=$name.oracle.stderr
    test -n "$o_es"     || o_es=0
    tf_debug "t_cmd='$t_cmd'"
    tf_debug "t_desc='$t_desc'"
    tf_debug "t_env='$t_env'"
    tf_debug "o_stdout='$o_stdout'"
    tf_debug "o_stderr='$o_stderr'"
    tf_debug "o_es='$o_es'"
    local ses=0
    if $setup;
    then
        local tcmd="
            tail -n+$t_start_at $name \\
              | $t_env $t_cmd > $name.result.stdout \\
                2> $name.result.stderr
        "
        tf_debug "tcmd='$tcmd'"
        eval "$tcmd"; r_es=$?
        if ! test $o_es -eq $r_es;
        then
            warn "exit status was $r_es, not $o_es"
            ses=$TF_ES_FAIL
        fi
        diff -u $name.oracle.stderr $name.result.stderr || ses=$TF_ES_FAIL
        diff -u $name.oracle.stdout $name.result.stdout || ses=$TF_ES_FAIL
    else
        tf_warn "setup phase failed, skipping: $name"
    fi
    if ! $cleanup;
    then
        tf_warn "cleanup phase failed: $name"
        ses=$TF_ES_PANIC
    fi
    return $ses
}

tf_do_subtests() {
    local subtname es tes
    es=0
    for subtname in $(tf_enum_subtests | grep -e "$TF_FILTER_SUBTEST");
    do
        export TF_SUBTNAME=$subtname
        tf_think "::: $TF_TNAME::$TF_SUBTNAME"
        tf_do_subtest "$TF_SUBTNAME";
        tes=$?
        test $tes -gt $es && es=$tes
        test $tes -gt 0 && tf_warn "!!! $TF_TNAME::$TF_SUBTNAME ($tes)"
    done
    exit $es
}
